/*
 * Copyright (c) 2015 Tricoire Sebastien 3dsman@free.fr
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software
 * in a product, an acknowledgment in the product documentation would be
 * appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 */

#include "stitchs/OE_fillstitch.h"
#include "curves/OE_pointcurve.h"
#include "curves/OE_joincurve.h"
#include "curves/OE_subcurve.h"
#include "OE_utils.h"
#include <iostream>
#include <stdio.h>
#include <algorithm>

void OE_fillstitch::contour_segment::addIntersection(OE_fillstitch::intersection intersection)
{
	intersections.push_back(intersection);
	if (minXCell == -1)
	{
		minXCell = intersection.gridXPos;
		maxXCell = intersection.gridXPos;
		minYCell = intersection.gridYPos;
		maxYCell = intersection.gridYPos;
	}
	else
	{
		minXCell = mini(minXCell,intersection.gridXPos);
		maxXCell = maxi(maxXCell,intersection.gridXPos);
		minYCell = mini(minYCell,intersection.gridYPos);
		maxYCell = maxi(maxYCell,intersection.gridYPos);
	}
}

OE_fillstitch::fillPatch::fillPatch() : minXCell(-1)
{
}

void OE_fillstitch::fillPatch::addSegment(contour_segment _segment, int _id)
{
	contour_segments.push_back(_id);
	if (minXCell == -1)
	{
		minXCell = _segment.minXCell;
		minYCell = _segment.minYCell;
		maxXCell = _segment.maxXCell;
		maxYCell = _segment.maxYCell;
	}
	else
	{
		minXCell = mini(minXCell,_segment.minXCell);
		minYCell = mini(minYCell,_segment.minYCell);
		maxXCell = maxi(maxXCell,_segment.maxXCell);
		maxYCell = maxi(maxYCell,_segment.maxYCell);
	}
}

void OE_fillstitch::fillPatch::addCut(int _cut)
{
	cuts.push_back(_cut);
}

OE_fillstitch::cut::cut(int _intersect1, int _intersect2) : contours(4, -1)
{
	intersect1 = _intersect1;
	intersect2 = _intersect2;
}

OE_fillstitch::OE_fillstitch() : curve(nullptr), reverse(false), len(1), width(1), pattern(nullptr)
{
}

OE_fillstitch::OE_fillstitch(OE_thread* thread, float len, float width, OE_pattern* pattern)
{
	curve = new OE_joincurve();
	curve->addDependency(this);
	curve->setClosed(true);
	this->len = len;
	this->width = width;
	this->thread = thread;
	this->pattern = pattern;
	this->maxLen = 5;
	setNeedRefresh();
}

void OE_fillstitch::persist(Pakal::Archive* archive)
{
	OE_stitchs::persist(archive);

	archive->set_type_name("OE_fillstitch");
	archive->value("curve", curve);
	archive->value("reverse", reverse);
	archive->value("len", len);
	archive->value("width", width);
	//archive->value("pattern", pattern);
}


OE_fillstitch::~OE_fillstitch()
{
	std::list<OE_subcurve*>::iterator i = curve->curves.begin();
	 while (i != curve->curves.end())
	 {
		 (*i)->removeDependency(curve);
		 delete *i;
		 i++;
	 }
	if (curve)
	{
		delete curve;
	}
}

/*
void  OE_fillstitch::setCurve(OE_curve* curve)
{
	if (this->curve)this->curve->removeDependency(this);
	this->curve = curve;
	curve->addDependency(this);
	setNeedRefresh();
}
*/

void  OE_fillstitch::setLen(float len)
{
	this->len = len;
	setNeedRefresh();
}
void  OE_fillstitch::setWidth(float width)
{
	this->width = width;
	setNeedRefresh();
}
void  OE_fillstitch::setPattern(OE_pattern* stitch)
{
	pattern=stitch;
	setNeedRefresh();
}

OE_joincurve* OE_fillstitch::getJoincurve()
{
	return curve;
}
float OE_fillstitch::getLen()
{
	return len;
}
float OE_fillstitch::getWidth()
{
	return width;
}
OE_pattern* OE_fillstitch::getPattern()
{
	return pattern;
}

bool OE_fillstitch::check()
{
	return curve && curve->check() && pattern && len;
}

void OE_fillstitch::initGridControl()
{

	BoundingBox bound;
	vector_2d min;
	vector_2d max;

	bound = curve->getBound();

	min = bound.getMin();
	max = bound.getMax();
	min = min + vector_2d(-1, -1);
	max = max + vector_2d(1, 1);

	gridControl.resize(16);
	gridControl[0]=min;
	gridControl[1]=vector_2d(min.x+(max.x-min.x)/3, min.y-20);
	gridControl[2]=vector_2d(min.x+2*(max.x-min.x)/3, min.y+1);
	gridControl[3]=vector_2d(max.x, min.y);

	gridControl[12]=vector_2d(min.x, max.y);
	gridControl[13]=vector_2d(min.x+(max.x-min.x)/3, max.y+5);
	gridControl[14]=vector_2d(min.x+2*(max.x-min.x)/3, max.y+12);
	gridControl[15]=max;

	gridControl[4]=(gridControl[12]-gridControl[0])/3+gridControl[0];
	gridControl[5]=(gridControl[13]-gridControl[1])/3+gridControl[0];
	gridControl[6]=(gridControl[14]-gridControl[2])/3+gridControl[0];
	gridControl[7]=(gridControl[15]-gridControl[3])/3+gridControl[0];

	gridControl[8]=(gridControl[12]-gridControl[0])*2/3+gridControl[0];
	gridControl[9]=(gridControl[13]-gridControl[1])*2/3+gridControl[0];
	gridControl[10]=(gridControl[14]-gridControl[2])*2/3+gridControl[0];
	gridControl[11]=(gridControl[15]-gridControl[3])*2/3+gridControl[0];

}

std::vector<std::vector<vector_2d>> OE_fillstitch::generateGrid(float precision)
{
	OE_pointcurve curve1;
	OE_pointcurve curve2;
	OE_pointcurve curve3;
	OE_pointcurve curve4;
	std::vector<std::vector<vector_2d>> gridPts;

	initGridControl();

	curve1.addPoint(gridControl[0]);
	curve1.addPoint(gridControl[1]);
	curve1.addPoint(gridControl[2]);
	curve1.addPoint(gridControl[3]);

	curve2.addPoint(gridControl[12]);
	curve2.addPoint(gridControl[13]);
	curve2.addPoint(gridControl[14]);
	curve2.addPoint(gridControl[15]);

	curve3.addPoint(gridControl[0]);
	curve3.addPoint(gridControl[4]);
	curve3.addPoint(gridControl[8]);
	curve3.addPoint(gridControl[12]);

	curve4.addPoint(gridControl[3]);
	curve4.addPoint(gridControl[5]);
	curve4.addPoint(gridControl[7]);
	curve4.addPoint(gridControl[15]);

	float curve1Len = curve1.getLength(precision);
	float curve2Len = curve2.getLength(precision);
	float curve3Len = curve3.getLength(precision);
	float curve4Len = curve4.getLength(precision);
	gridXSize = (curve1Len+curve2Len)/2/len;
	gridYSize = (curve3Len+curve4Len)/2/width;

	std::cout<<"gridSize: "<<gridXSize<<" "<<gridYSize<<std::endl;
	std::vector<vector_2dt> curve1Pts;
	std::vector<vector_2dt> curve2Pts;
	curve1Pts = curve1.discretizeRegular(curve1Len/gridXSize);
	curve2Pts = curve2.discretizeRegular(curve2Len/gridXSize);

	gridPts.resize(gridXSize+1);
	for (unsigned i=0; i<gridXSize+1; ++i)
	{
		gridPts[i].resize(gridYSize+1);
	}

	vector_2d pt;
	for(unsigned i=0; i<=gridXSize; i++)
	{
		for(unsigned j=0; j<=gridYSize; j++)
		{
			gridPts[i][j] = curve2Pts.at(i)+(curve1Pts.at(i)-curve2Pts.at(i))/gridXSize*j;
		}
	}
	return gridPts;
}

bool OE_fillstitch::findIntersections(const std::vector<vector_2dt> curvePts, const std::vector<std::vector<vector_2d>> gridPts, std::vector<OE_fillstitch::intersection> &intersections, std::vector<int> &inflexions)
{
	// must find and init to the start cell
	int curXCell=0;
	int curYCell=0;
	bool out = false;
	do
	{
		curYCell = 0;
		do
		{
			//we check every cell to find the one where the first curve point is
			out = (PointInTriangle(curvePts[0],
								   gridPts[curXCell+1][curYCell],
								   gridPts[curXCell][curYCell],
								   gridPts[curXCell][curYCell+1]) ||
				   PointInTriangle(curvePts[0],
								   gridPts[curXCell+1][curYCell],
								   gridPts[curXCell][curYCell+1],
								   gridPts[curXCell+1][curYCell+1]));
			curYCell++;
		} while (curYCell<gridYSize && !out);
		curXCell++;
	} while (curXCell<gridXSize && !out);
	curXCell--;
	curYCell--;

	// find every intersection between the curve and the grid
	char dir = 0;

	bool intersected = false;
	int prevP = curvePts.size()-1;

	vector_2d intersectPt;
	float intersectPosGrid;
	float intersectPosCurve;
	bool up = false;

	for(std::vector<vector_2dt>::size_type p = 0; p < curvePts.size(); p++)
	{
		do
		{
			intersected = false;
			// check if the segment get out the cell and change the cell in consequence
			// dir store the incoming direction of the segment in a cell. That reduce testing and avoid double detection
			if (dir!=1 && intersect(curvePts[prevP].v, curvePts[p].v,
						  gridPts[curXCell][curYCell], gridPts[curXCell][curYCell+1],
						  intersectPt, intersectPosCurve, intersectPosGrid))
			{
				intersections.push_back(intersection(intersectPt, false, curYCell + intersectPosCurve, p, curXCell));
				dir = 2;
				curXCell--;
				if (curXCell<0)
				{
					return false;
				}
				intersected = true;
			}
			if (dir!=2 && intersect(curvePts[prevP].v, curvePts[p].v,
						  gridPts[curXCell+1][curYCell], gridPts[curXCell+1][curYCell+1],
						  intersectPt, intersectPosCurve, intersectPosGrid))
			{
				curXCell++;
				if (curXCell>=gridXSize)
				{
					return false;
				}
				intersections.push_back(intersection(intersectPt, false, curYCell + intersectPosCurve, p, curXCell));
				dir = 1;
				intersected = true;
			}
			if (dir!=3 && intersect(curvePts[prevP].v, curvePts[p].v,
						  gridPts[curXCell][curYCell], gridPts[curXCell+1][curYCell],
						  intersectPt, intersectPosCurve, intersectPosGrid))
			{
				intersections.push_back(intersection(intersectPt, true, curYCell, p, curXCell + intersectPosGrid));

				if (up)
				{
					// if needed we store inflexion point local maximum here
					inflexions.push_back(intersections.size()-1);
				}
				up = false;
				dir = 4;
				curYCell--;
				if (curYCell<0)
				{
					return false;
				}
				intersected = true;
			}
			if (dir!=4 && intersect(curvePts[prevP].v, curvePts[p].v,
						  gridPts[curXCell][curYCell+1], gridPts[curXCell+1][curYCell+1],
						  intersectPt, intersectPosCurve, intersectPosGrid))
			{
				curYCell++;
				if (curYCell>=gridYSize)
				{
					return false;
				}
				intersections.push_back(intersection(intersectPt, true, curYCell, p, curXCell + intersectPosGrid));

				if (!up)
				{
					// and local minimum here
					inflexions.push_back(intersections.size()-1);
				}
				up = true;
				dir = 3;
				intersected = true;
			}
		} while (intersected);
		dir = 0;
		prevP = p;
	}

	if (intersections.size() == 0)
	{
		return false;
	}
	return true;
}

bool OE_fillstitch::findCuts( std::vector<int> inflexions, std::vector<OE_fillstitch::intersection> &intersections, std::vector<cut> &cuts)
{
	int nbCross = 0;
	int closestLeft = -1;
	int closestRight = -1;

	//for each inflexion point i will find closest intersection on the same grid line
	for(int inflex : inflexions)
	{
		closestLeft = -1;
		closestRight = -1;
		int prevInflex;
		int prev = inflex;
		prevInflex = prev;

		//find the previous point that cross the same X grid line
		do
		{
			prevInflex = prevInflex-1%intersections.size();
		}
		while (intersections[prevInflex].xcross != true ||
			   intersections[prevInflex].gridYPos != intersections[inflex].gridYPos);

		for(std::vector<intersection>::size_type p = 0; p < intersections.size(); p++)
		{
			//check every intersection to find those who are on the same grid line
			if (intersections[p].xcross == true &&
				intersections[p].gridYPos == intersections[inflex].gridYPos &&
				p != inflex &&
				p != prevInflex)
			{
				// check if they are before or after the inflexion point, get the closest and count the one that are at left
				if (intersections[p].gridXPos < intersections[inflex].gridXPos)
				{
					if (closestLeft == -1 || intersections[closestLeft].gridXPos < intersections[p].gridXPos)
					{
						closestLeft = p;
					}
					nbCross++;
				}
				else
				{
					if (closestRight == -1 || intersections[closestRight].gridXPos > intersections[p].gridXPos)
					{
						closestRight = p;
					}
				}
			}
		}
		// if we didn't cross the curve twice, we're inside the curve
		if (nbCross%2 == 1)
		{
			//we add then the 2 cuts to the cut list and set the cutIndex to the intersections
			if (intersections[prevInflex].gridXPos < intersections[inflex].gridXPos)
			{

				intersections[closestLeft].cutIndex = cuts.size();
				intersections[prevInflex].cutIndex = cuts.size();
				cuts.push_back(cut(closestLeft,prevInflex));

				intersections[inflex].cutIndex = cuts.size();
				intersections[closestRight].cutIndex = cuts.size();
				cuts.push_back(cut(inflex,closestRight));
			}
			else
			{
				intersections[closestLeft].cutIndex = cuts.size();
				intersections[inflex].cutIndex = cuts.size();
				cuts.push_back(cut(closestLeft,inflex));

				intersections[prevInflex].cutIndex = cuts.size();
				intersections[closestRight].cutIndex = cuts.size();
				cuts.push_back(cut(prevInflex,closestRight));
			}
		}
		nbCross = 0;
	}
}

bool OE_fillstitch::findPatchs(const std::vector<OE_fillstitch::intersection> intersections, std::vector<cut> &cuts, std::vector<contour_segment> &segments, std::vector<fillPatch> &patchs)
{

	int startIntersect=0, curIntersect=0, curSegment = 0, curcut=0, curpatch = 0;

	std::list<int> patchsStack;
	//std::list<int> cutStack;
	patchs.push_back(fillPatch());
	patchsStack.push_front(0);

	if (!cuts.size())
	{
		while (curIntersect<intersections.size())
		{
			segments[0].addIntersection(intersections[curIntersect]);
			curIntersect++;
		}
		patchs[0].addSegment(segments[0], 0);
		return true;
	}


	//loop to start at the first cut
	while (intersections[startIntersect].cutIndex == -1 && startIntersect<intersections.size())
	{
		startIntersect++;
	}
	curcut = intersections[startIntersect].cutIndex;
	//set the cutStart of the first segment with the first cut index
	segments[0].cutStart = curcut;
	//cutStack.push_front(curcut);
	cuts[curcut].contours[0] = curSegment;


	//add the first intersection to the first segment
	segments[curSegment].addIntersection(intersections[startIntersect]);
	curIntersect = startIntersect;


	//loop on every intersections
	do{

		//increment the current intersection
		curIntersect = (curIntersect+1)%intersections.size();

		//add the curent intersection to the current segment
		segments[curSegment].addIntersection(intersections[curIntersect]);
		//if we find a cut
		if (intersections[curIntersect].cutIndex!=-1)
		{
			//we set it as current
			curcut = intersections[curIntersect].cutIndex;

			//we close the current segment by setting cutEnd and add it to the current patch
			segments[curSegment].cutEnd = curcut;
			patchs[curpatch].addSegment(segments[curSegment], segments.size()-1);

			if ((curIntersect != startIntersect))
			{
				//create a new segment
				segments.push_back(contour_segment());
				curSegment++;

				segments[curSegment].cutStart = curcut;

				//add the current intersection to the new segment
				segments[curSegment].addIntersection(intersections[curIntersect]);

				// if we already marked this cut we can pop the patch out of the stack and mark contours indexes
				if (cuts[curcut].contours[0]!=-1)
				{
					cuts[curcut].contours[1] = curSegment-1;
					cuts[curcut].contours[3] = curSegment;
					patchsStack.pop_front();

					// if the stack is empty we start a new pach
					if (patchsStack.empty())
					{
						patchs.push_back(fillPatch());
						patchsStack.push_front(patchs.size()-1);
					}
				}
				else
				{
					//else it's the first contact with this cut so we mark his first contours index
					cuts[curcut].contours[0] = curSegment;
					cuts[curcut].contours[2] = curSegment-1;
					//then init a new patch
					patchs.push_back(fillPatch());
					patchsStack.push_front(patchs.size()-1);
				}
				curpatch = patchsStack.front();
			}
			else
			{
				//if we finished the loop we just need to set the last cut contour
				cuts[curcut].contours[2] = curSegment-1;
			}
		}

	}while (curIntersect!=startIntersect);

	return true;
}

bool OE_fillstitch::generateStitchLines(std::vector<cut> cuts, std::vector<contour_segment> segments, fillPatch &patch)
{
	patch.stitch_lines.resize(patch.maxYCell-patch.minYCell+1);
	for(std::vector<contour_segment>::size_type segment = 0; segment < patch.contour_segments.size(); segment++)
	{
		//std::cout<<"segmentID "<< patch.contour_segments[segment]<<std::endl;
		const contour_segment curSegment = segments[patch.contour_segments[segment]];
		//std::cout<<" intersections "<<curSegment.intersections.size()<<std::endl;
		for(std::vector<intersection>::size_type p = 0; p < curSegment.intersections.size(); p++)
		{
			int yCell = curSegment.intersections[p].gridYPos;
			int stitch_line_id = yCell - patch.minYCell;
			patch.stitch_lines[stitch_line_id].yCell = yCell;
			stitch_line* curStitch_line = &(patch.stitch_lines[yCell - patch.minYCell]);
			int xCell = curSegment.intersections[p].gridXPos;
			//std::cout<<"  lineID "<< stitch_line_id<<"  xcell "<<xCell<<"  ycell "<<yCell<<std::endl;
			if ((curStitch_line->minXCell>xCell)||(curStitch_line->minXCell == -1))
			{
				curStitch_line->minXCell=xCell;
			}
			if ((curStitch_line->maxXCell<xCell)||(curStitch_line->maxXCell == -1))
			{
				curStitch_line->maxXCell=xCell;
			}
		}
		//std::cout<<"segmentID "<< patch.contour_segments[segment]<<" ok"<<std::endl;
	}
	//check if results is consistent (can be incorrect in case of self intersecting form)
	for(std::vector<stitch_line>::size_type curLine = 0; curLine < patch.stitch_lines.size(); curLine++)
	{
		stitch_line line = patch.stitch_lines[curLine];
		if ((line.minXCell<0)||(line.maxXCell<0))
		{
			return false;
		}
	}
	return true;
}

void OE_fillstitch::refresh(float dpi, bool force)
{
	std::vector<vector_2dt> curvePts;
	std::vector<vector_2dt> prevline;
	std::vector<vector_2dt> curline;

	std::vector<std::vector<vector_2d>> gridPts;
	std::vector<intersection> intersections;
	std::vector<int> inflexion;
	std::vector<cut> cuts;
	std::vector<contour_segment> segments (1);
	std::vector<fillPatch> patchs;

	std::vector<vector_2d> outPts;

	unsigned int x,y = 0;
	if (!check() || !(needRefresh && force))
	{
		return;
	}

	curve->setClosed(true);
	curve->refresh( dpi);
	if (curve->getNpts()<=3)
	{
		return;
	}

	curvePts = curve->discretizeFast(0.1);

	if (curvePts.size()<2)
	{
		std::cout << "curve discretisation fail" << std::endl;
		return;
	}

	gridPts = generateGrid(dpi);

	/*
	for(unsigned i=0; i<=gridXSize; i++)
	{
		for(unsigned j=0; j<=gridYSize; j++)
		{
			outPts.push_back(gridPts[i][j]);
		}
	}*/

	std::cout << "findIntersections" << std::endl;
	if (!findIntersections(curvePts, gridPts, intersections, inflexion))
	{
		std::cout << "problem in findIntersections" << std::endl;
		return;
	}
	std::cout << "findCuts" << std::endl;
	if (!findCuts(inflexion, intersections, cuts))
	{
		std::cout << "problem in findCuts" << std::endl;
		return;
	}

	std::cout << "findPatchs" << std::endl;
	if (!findPatchs( intersections, cuts, segments, patchs))
	{
		std::cout << "problem in findPatchs" << std::endl;
		return;
	}

	std::cout << "generateStitchLines" << std::endl;
	for(std::vector<fillPatch>::size_type curPatch = 0; curPatch < patchs.size(); curPatch++)
	{
		if (!generateStitchLines( cuts, segments, patchs[curPatch]))
		{
			std::cout << "problem in generateStitchLines" << std::endl;
			pts.clear();
			return;
		}
		std::cout << "generateStitchLines ok" << std::endl;
	}
	// -----------  show result  --------------

	std::cout<<"show"<<std::endl;
	outPts.clear();

#if (0)
	for(std::vector<cut>::size_type curcut = 0; curcut < cuts.size(); curcut++)
	{
		outPts.push_back(intersections[cuts[curcut].intersect1].pt);
		outPts.push_back(intersections[cuts[curcut].intersect2].pt);
		outPts.push_back(vector_2d());
	}
#endif
	int curCut = 0;
	int cursegment = -1;
	int curPatch = 0;

#if (0)
	for(std::vector<fillPatch>::size_type curPatch = 0; curPatch < patchs.size(); curPatch++)
	{
		for(std::vector<contour_segment>::size_type segment = 0; segment < patchs[curPatch].contour_segments.size(); segment++)
		{
			for(std::vector<intersection>::size_type p = 0; p < segments[patchs[curPatch].contour_segments[segment]].intersections.size(); p++)
			{
				outPts.push_back(segments[patchs[curPatch].contour_segments[segment]].intersections[p].pt);
			}
		}
		outPts.push_back(segments[patchs[curPatch].contour_segments[0]].intersections[0].pt);
	}
#endif

#if (1)
	std::vector<vector_2d> linePts;
	std::vector<vector_2d> patchPts;
	stitch_line line;
	bool invertRow = false;
//	int curPatch = 0;
	for(std::vector<fillPatch>::size_type curPatch = 0; curPatch < patchs.size(); curPatch++)
	{
		invertRow = patchs[curPatch].invertRow;
		for(std::vector<stitch_line>::size_type curLine = 0; curLine < patchs[curPatch].stitch_lines.size(); curLine++)
		{
			line = patchs[curPatch].stitch_lines[curLine];

			for(unsigned j=line.minXCell; j<=line.maxXCell+1; j++)
			{
				linePts.push_back(gridPts[j][line.yCell]);
			}
			if (invertRow)
			{
				std::reverse(linePts.begin(), linePts.end());
			}
			invertRow = !invertRow;
			patchPts.insert(patchPts.end(), linePts.begin(), linePts.end());
			linePts.clear();
		}
		if (patchs[curPatch].invertCol)
		{
			std::reverse(patchPts.begin(), patchPts.end());
		}
		outPts.insert(outPts.end(), patchPts.begin(), patchPts.end());
		patchPts.clear();
	}
	/*for(std::vector<stitch_line>::size_type curLine = 0; curLine < patchs[0].stitch_lines.size(); curLine++)
	{
		line = patchs[0].stitch_lines[curLine];
		std::cout<<"miny "<<line.minXCell<<"maxy "<<line.maxXCell<<std::endl;
		for(unsigned j=line.minXCell; j<=line.maxXCell; j++)
		{
			outPts.push_back(gridPts[line.yCell][j]);
		}
	}*/
#endif

	pts.clear();

	if(maxLen>0)
		pts = stitchMaxLen(outPts, maxLen);
	else
		pts = outPts;

	needRefresh = false;
}

void OE_fillstitch::delDependency(OE_base * object)
{
	if (curve == object)
	{
		curve = 0;
	}
}

void OE_fillstitch::refreshDependency()
{
	if (curve)
	{
		curve->addDependency(this);
		curve->refreshDependency();
	}
}

